/*
 * Copyright 2009 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.cache;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;
import net.sf.ehcache.Statistics;

import dk.vajhoej.isam.Key;

/**
 * Class StandardIsamCache is a wrapper for EHCache for CacheIsamSource.
 */
public class EHCacheWrapper implements IsamCache {
	private static class CacheKey<T,TK> {
		private Class<T> clz;
		private Key<TK> k;
		public CacheKey(Class<T> clz, Key<TK> k) {
			this.clz = clz;
			this.k = k;
		}
		@Override
		public boolean equals(Object o) {
			if(o instanceof CacheKey<?,?>) {
				@SuppressWarnings("unchecked")
				CacheKey<T,TK> o2 = (CacheKey<T,TK>)o;
				return clz.equals(o2.clz)
				       && k.getIx() == o2.k.getIx()
				       && k.getValue().equals(o2.k.getValue());
			} else {
				return false;
			}
		}
		@Override
		public int hashCode() {
			return clz.hashCode() ^ k.getIx() ^ k.getValue().hashCode();
		}
		public static <T,TK> CacheKey<T,TK> create(Class<T> clz, Key<TK> k) {
			return new CacheKey<T,TK>(clz, k);
		}
	}
	private CacheManager mgr;
	private String cachnam;
	/**
	 * Create instance of StandardIsamCache. 
	 */
	public EHCacheWrapper(CacheManager mgr, String cachnam) {
	    this.mgr = mgr;
	    this.cachnam = cachnam;
	}
    @SuppressWarnings("unchecked")
	public synchronized <T, TK> T get(Class<T> t, Key<TK> k) {
        Cache c = mgr.getCache(cachnam);
        Element elm = c.get(new CacheKey(t, k));
        if(elm != null) {
            return (T)elm.getObjectValue();
        } else {
            return null;
        }
	}
	public synchronized <T, TK> void put(Key<TK> k, T o) {
        Cache c = mgr.getCache(cachnam);
        c.put(new Element(CacheKey.create(o.getClass(), k), o));
	}
    @SuppressWarnings("unchecked")
    public <T,TK> void remove(Class<T> clz,Key<TK> k) {
        Cache c = mgr.getCache(cachnam);
        c.remove(new CacheKey(clz, k));
    }
	public double getHitRate() {
        Cache c = mgr.getCache(cachnam);
        Statistics stat = c.getStatistics();
        if(stat.getCacheHits() + stat.getCacheMisses() > 0) {
            return stat.getCacheHits() * 1.0/ (stat.getCacheHits() + stat.getCacheMisses());
        } else {
            return 0;
        }
	}
}
