/*
 * Copyright 2009 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.rms.test;

import static org.junit.Assert.*;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.StringReader;

import org.junit.Test;

import dk.vajhoej.vms.rms.CParse;

public class TestCParse {
    @Test
    public void testSimple() {
        String src = "struct data1\n" +
                     "{\n" +
                     "    int a;\n" +
                     "    double b;\n" +
                     "};\n";
        String out = "import java.io.Serializable;\n" +
                     "\n" +
                     "import dk.vajhoej.isam.KeyField;\n" +
                     "import dk.vajhoej.record.Alignment;\n" +
                     "import dk.vajhoej.record.FieldType;\n" +
                     "import dk.vajhoej.record.Struct;\n" +
                     "import dk.vajhoej.record.StructField;\n" +
                     "\n" +
                     "@Struct(alignment=Alignment.NATURAL,endpad=true)\n" +
                     "public class Data1 implements Serializable {\n" +
                     "    private static final long serialVersionUID = 1L;\n" +
                     "    @KeyField(n=0)\n" +
                     "    @StructField(n=0,type=FieldType.INT4)\n" +
                     "    private int a;\n" +
                     "    @StructField(n=1,type=FieldType.FP8)\n" +
                     "    private double b;\n" +
                     "    public int getA() {\n" +
                     "        return a;\n" +
                     "    }\n" +
                     "    public void setA(int a) {\n" +
                     "        this.a = a;\n" +
                     "    }\n" +
                     "    public double getB() {\n" +
                     "        return b;\n" +
                     "    }\n" +
                     "    public void setB(double b) {\n" +
                     "        this.b = b;\n" +
                     "    }\n" +
                     "}\n";
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(baos);
            PrintStream sav = System.out;
            System.setOut(ps);
            CParse.processFile(src);
            ps.close();
            System.setOut(sav);
            String[] expect = out.split("\n");
            StringReader sr = new StringReader(new String(baos.toByteArray()));
            BufferedReader br = new BufferedReader(sr);
            for(int i = 0; i < expect.length; i++) {
                String line = br.readLine();
                assertEquals(expect[i], line);
            }
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testComplex() {
        String src = "program complex;\n" + 
        "\n" + 
                     "typedef char str8[8];\n" +
                     "typedef char str255[255];\n" +
                     "\n" + 
                     "struct data1\n" +
                     "{\n" +
                     "    int a;\n" +
                     "    double b;\n" +
                     "};\n" +
                     "\n" + 
                     "struct data2\n" +
                     "{\n" +
                     "    int x;\n" +
                     "    str8 y;\n" +
                     "    str255 z;\n" +
                     "    int w1:1;\n" +
                     "    int w2:31;\n" +
                     "};\n";
        String out = "import java.io.Serializable;\n" +
                     "\n" +
                     "import dk.vajhoej.isam.KeyField;\n" +
                     "import dk.vajhoej.record.Alignment;\n" +
                     "import dk.vajhoej.record.FieldType;\n" +
                     "import dk.vajhoej.record.Struct;\n" +
                     "import dk.vajhoej.record.StructField;\n" +
                     "\n" +
                     "@Struct(alignment=Alignment.NATURAL,endpad=true)\n" +
                     "public class Data1 implements Serializable {\n" +
                     "    private static final long serialVersionUID = 1L;\n" +
                     "    @KeyField(n=0)\n" +
                     "    @StructField(n=0,type=FieldType.INT4)\n" +
                     "    private int a;\n" +
                     "    @StructField(n=1,type=FieldType.FP8)\n" +
                     "    private double b;\n" +
                     "    public int getA() {\n" +
                     "        return a;\n" +
                     "    }\n" +
                     "    public void setA(int a) {\n" +
                     "        this.a = a;\n" +
                     "    }\n" +
                     "    public double getB() {\n" +
                     "        return b;\n" +
                     "    }\n" +
                     "    public void setB(double b) {\n" +
                     "        this.b = b;\n" +
                     "    }\n" +
                     "}\n" +
                     "\n" +
                     "import java.io.Serializable;\n" +
                     "\n" +
                     "import dk.vajhoej.isam.KeyField;\n" +
                     "import dk.vajhoej.record.Alignment;\n" +
                     "import dk.vajhoej.record.FieldType;\n" +
                     "import dk.vajhoej.record.Struct;\n" +
                     "import dk.vajhoej.record.StructField;\n" +
                     "\n" +
                     "@Struct(alignment=Alignment.NATURAL,endpad=true)\n" +
                     "public class Data2 implements Serializable {\n" +
                     "    private static final long serialVersionUID = 1L;\n" +
                     "    @KeyField(n=0)\n" +
                     "    @StructField(n=0,type=FieldType.INT4)\n" +
                     "    private int x;\n" +
                     "    @StructField(n=1,type=FieldType.FIXSTRNULTERM,length=8)\n" +
                     "    private String y;\n" +
                     "    @StructField(n=2,type=FieldType.FIXSTRNULTERM,length=255)\n" +
                     "    private String z;\n" +
                     "    @StructField(n=3,type=FieldType.BIT,length=1)\n" +
                     "    private int w1;\n" +
                     "    @StructField(n=4,type=FieldType.BIT,length=31)\n" +
                     "    private int w2;\n" +
                     "    public int getX() {\n" +
                     "        return x;\n" +
                     "    }\n" +
                     "    public void setX(int x) {\n" +
                     "        this.x = x;\n" +
                     "    }\n" +
                     "    public String getY() {\n" +
                     "        return y;\n" +
                     "    }\n" +
                     "    public void setY(String y) {\n" +
                     "        this.y = y;\n" +
                     "    }\n" +
                     "    public String getZ() {\n" +
                     "        return z;\n" +
                     "    }\n" +
                     "    public void setZ(String z) {\n" +
                     "        this.z = z;\n" +
                     "    }\n" +
                     "    public int getW1() {\n" +
                     "        return w1;\n" +
                     "    }\n" +
                     "    public void setW1(int w1) {\n" +
                     "        this.w1 = w1;\n" +
                     "    }\n" +
                     "    public int getW2() {\n" +
                     "        return w2;\n" +
                     "    }\n" +
                     "    public void setW2(int w2) {\n" +
                     "        this.w2 = w2;\n" +
                     "    }\n" +
                     "}\n";
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(baos);
            PrintStream sav = System.out;
            System.setOut(ps);
            CParse.processFile(src);
            ps.close();
            System.setOut(sav);
            String[] expect = out.split("\n");
            StringReader sr = new StringReader(new String(baos.toByteArray()));
            BufferedReader br = new BufferedReader(sr);
            for(int i = 0; i < expect.length; i++) {
                String line = br.readLine();
                assertEquals(expect[i], line);
            }
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testUnusal() {
        String src = "struct data1\n" +
                     "{\n" +
                     "    unsigned int a;\n" +
                     "    signed short int b;\n" +
                     "    unsigned short int c;\n" +
                     "    signed char d;\n" +
                     "    unsigned char e;\n" +
                     "};\n";
        String out = "import java.io.Serializable;\n" +
                     "\n" +
                     "import dk.vajhoej.isam.KeyField;\n" +
                     "import dk.vajhoej.record.Alignment;\n" +
                     "import dk.vajhoej.record.FieldType;\n" +
                     "import dk.vajhoej.record.Struct;\n" +
                     "import dk.vajhoej.record.StructField;\n" +
                     "\n" +
                     "@Struct(alignment=Alignment.NATURAL,endpad=true)\n" +
                     "public class Data1 implements Serializable {\n" +
                     "    private static final long serialVersionUID = 1L;\n" +
                     "    @KeyField(n=0)\n" +
                     "    @StructField(n=0,type=FieldType.UINT4)\n" +
                     "    private long a;\n" +
                     "    @StructField(n=1,type=FieldType.INT2)\n" +
                     "    private short b;\n" +
                     "    @StructField(n=2,type=FieldType.UINT2)\n" +
                     "    private int c;\n" +
                     "    @StructField(n=3,type=FieldType.INT1)\n" +
                     "    private byte d;\n" +
                     "    @StructField(n=4,type=FieldType.UINT1)\n" +
                     "    private short e;\n" +
                     "    public long getA() {\n" +
                     "        return a;\n" +
                     "    }\n" +
                     "    public void setA(long a) {\n" +
                     "        this.a = a;\n" +
                     "    }\n" +
                     "    public short getB() {\n" +
                     "        return b;\n" +
                     "    }\n" +
                     "    public void setB(short b) {\n" +
                     "        this.b = b;\n" +
                     "    }\n" +
                     "    public int getC() {\n" +
                     "        return c;\n" +
                     "    }\n" +
                     "    public void setC(int c) {\n" +
                     "        this.c = c;\n" +
                     "    }\n" +
                     "    public byte getD() {\n" +
                     "        return d;\n" +
                     "    }\n" +
                     "    public void setD(byte d) {\n" +
                     "        this.d = d;\n" +
                     "    }\n" +
                     "    public short getE() {\n" +
                     "        return e;\n" +
                     "    }\n" +
                     "    public void setE(short e) {\n" +
                     "        this.e = e;\n" +
                     "    }\n" +
                     "}\n";
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(baos);
            PrintStream sav = System.out;
            System.setOut(ps);
            CParse.processFile(src);
            ps.close();
            System.setOut(sav);
            String[] expect = out.split("\n");
            StringReader sr = new StringReader(new String(baos.toByteArray()));
            BufferedReader br = new BufferedReader(sr);
            for(int i = 0; i < expect.length; i++) {
                String line = br.readLine();
                assertEquals(expect[i], line);
            }
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testInline() {
        String src = "struct data1\n" +
                     "{\n" +
                     "    int a;\n" +
                     "    char b[100];\n" +
                     "};\n";
        String out = "import java.io.Serializable;\n" +
                     "\n" +
                     "import dk.vajhoej.isam.KeyField;\n" +
                     "import dk.vajhoej.record.Alignment;\n" +
                     "import dk.vajhoej.record.FieldType;\n" +
                     "import dk.vajhoej.record.Struct;\n" +
                     "import dk.vajhoej.record.StructField;\n" +
                     "\n" +
                     "@Struct(alignment=Alignment.NATURAL,endpad=true)\n" +
                     "public class Data1 implements Serializable {\n" +
                     "    private static final long serialVersionUID = 1L;\n" +
                     "    @KeyField(n=0)\n" +
                     "    @StructField(n=0,type=FieldType.INT4)\n" +
                     "    private int a;\n" +
                     "    @StructField(n=1,type=FieldType.FIXSTRNULTERM,length=100)\n" +
                     "    private String b;\n" +
                     "    public int getA() {\n" +
                     "        return a;\n" +
                     "    }\n" +
                     "    public void setA(int a) {\n" +
                     "        this.a = a;\n" +
                     "    }\n" +
                     "    public String getB() {\n" +
                     "        return b;\n" +
                     "    }\n" +
                     "    public void setB(String b) {\n" +
                     "        this.b = b;\n" +
                     "    }\n" +
                     "}\n";
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(baos);
            PrintStream sav = System.out;
            System.setOut(ps);
            CParse.processFile(src);
            ps.close();
            System.setOut(sav);
            String[] expect = out.split("\n");
            StringReader sr = new StringReader(new String(baos.toByteArray()));
            BufferedReader br = new BufferedReader(sr);
            for(int i = 0; i < expect.length; i++) {
                String line = br.readLine();
                assertEquals(expect[i], line);
            }
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testComments() {
        String src = "struct /* bla */ data1 /* bla */\n" +
                     "{\n" +
                     "    int /* bla */ a /* bla */; /* bla */\n" +
                     "    double /* bla */ b /* bla */; /* bla */\n" +
                     "}; /* bla */\n";
        String out = "import java.io.Serializable;\n" +
                     "\n" +
                     "import dk.vajhoej.isam.KeyField;\n" +
                     "import dk.vajhoej.record.Alignment;\n" +
                     "import dk.vajhoej.record.FieldType;\n" +
                     "import dk.vajhoej.record.Struct;\n" +
                     "import dk.vajhoej.record.StructField;\n" +
                     "\n" +
                     "@Struct(alignment=Alignment.NATURAL,endpad=true)\n" +
                     "public class Data1 implements Serializable {\n" +
                     "    private static final long serialVersionUID = 1L;\n" +
                     "    @KeyField(n=0)\n" +
                     "    @StructField(n=0,type=FieldType.INT4)\n" +
                     "    private int a;\n" +
                     "    @StructField(n=1,type=FieldType.FP8)\n" +
                     "    private double b;\n" +
                     "    public int getA() {\n" +
                     "        return a;\n" +
                     "    }\n" +
                     "    public void setA(int a) {\n" +
                     "        this.a = a;\n" +
                     "    }\n" +
                     "    public double getB() {\n" +
                     "        return b;\n" +
                     "    }\n" +
                     "    public void setB(double b) {\n" +
                     "        this.b = b;\n" +
                     "    }\n" +
                     "}\n";
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(baos);
            PrintStream sav = System.out;
            System.setOut(ps);
            CParse.processFile(src);
            ps.close();
            System.setOut(sav);
            String[] expect = out.split("\n");
            StringReader sr = new StringReader(new String(baos.toByteArray()));
            BufferedReader br = new BufferedReader(sr);
            for(int i = 0; i < expect.length; i++) {
                String line = br.readLine();
                assertEquals(expect[i], line);
            }
        } catch (IOException e) {
            fail("Unexpected exception: " + e);
        }
    }
}
